@@ -50,23 +50,6 @@ G_DEFINE_TYPE (
 	e_week_view_main_item,
 	GNOME_TYPE_CANVAS_ITEM)
 
-static gint
-gdate_to_cal_weekdays (GDateWeekday wd)
-{
-	switch (wd) {
-	case G_DATE_MONDAY: return CAL_MONDAY;
-	case G_DATE_TUESDAY: return CAL_TUESDAY;
-	case G_DATE_WEDNESDAY: return CAL_WEDNESDAY;
-	case G_DATE_THURSDAY: return CAL_THURSDAY;
-	case G_DATE_FRIDAY: return CAL_FRIDAY;
-	case G_DATE_SATURDAY: return CAL_SATURDAY;
-	case G_DATE_SUNDAY: return CAL_SUNDAY;
-	default: break;
-	}
-
-	return 0;
-}
-
 static void
 week_view_main_item_draw_day (EWeekViewMainItem *main_item,
                               gint day,
@@ -78,22 +61,27 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
                               gint height)
 {
 	EWeekView *week_view;
+	ECalModel *model;
 	GtkStyle *style;
 	gint right_edge, bottom_edge, date_width, date_x, line_y;
 	gboolean show_day_name, show_month_name, selected;
 	gchar buffer[128], *format_string;
-	gint day_of_week, month, day_of_month, max_width;
+	gint month, day_of_month, max_width;
+	GDateWeekday weekday;
 	GdkColor *bg_color;
 	PangoFontDescription *font_desc;
 	PangoContext *pango_context;
 	PangoFontMetrics *font_metrics;
 	PangoLayout *layout;
 	gboolean today = FALSE;
-	CalWeekdays working_days;
+	gboolean multi_week_view;
 
 	week_view = e_week_view_main_item_get_week_view (main_item);
+	model = e_calendar_view_get_model (E_CALENDAR_VIEW (week_view));
 	style = gtk_widget_get_style (GTK_WIDGET (week_view));
 
+	multi_week_view = e_week_view_get_multi_week_view (week_view);
+
 	/* Set up Pango prerequisites */
 	font_desc = pango_font_description_copy (style->font_desc);
 	pango_context = gtk_widget_get_pango_context (GTK_WIDGET (week_view));
@@ -101,8 +89,8 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 		pango_context, font_desc,
 		pango_context_get_language (pango_context));
 
-	day_of_week = gdate_to_cal_weekdays (g_date_get_weekday (date));
 	month = g_date_get_month (date);
+	weekday = g_date_get_weekday (date);
 	day_of_month = g_date_get_day (date);
 	line_y = y + E_WEEK_VIEW_DATE_T_PAD +
 		PANGO_PIXELS (pango_font_metrics_get_ascent (font_metrics)) +
@@ -125,8 +113,6 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 			&& g_date_get_day (date) == tt.day;
 	}
 
-	working_days = calendar_config_get_working_days ();
-
 	/* Draw the background of the day. In the month view odd months are
 	 * one color and even months another, so you can easily see when each
 	 * month starts (defaults are white for odd - January, March, ... and
@@ -134,9 +120,9 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 	 * same color, the color used for the odd months in the month view. */
 	if (today)
 		bg_color = &week_view->colors[E_WEEK_VIEW_COLOR_TODAY_BACKGROUND];
-	else if ((working_days & day_of_week) == 0)
+	else if (!e_cal_model_get_work_day (model, weekday))
 		bg_color = &week_view->colors[E_WEEK_VIEW_COLOR_MONTH_NONWORKING_DAY];
-	else if (week_view->multi_week_view && (month % 2 == 0))
+	else if (multi_week_view && (month % 2 == 0))
 		bg_color = &week_view->colors[E_WEEK_VIEW_COLOR_EVEN_MONTHS];
 	else
 		bg_color = &week_view->colors[E_WEEK_VIEW_COLOR_ODD_MONTHS];
@@ -179,7 +165,7 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 				cr, &week_view->colors[E_WEEK_VIEW_COLOR_SELECTED]);
 		}
 
-		if (week_view->multi_week_view) {
+		if (multi_week_view) {
 			cairo_rectangle (
 				cr, x + 2, y + 1,
 				width - 5,
@@ -203,7 +189,7 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 	 * the 1st of each month, otherwise use "10". */
 	show_day_name = FALSE;
 	show_month_name = FALSE;
-	if (!week_view->multi_week_view) {
+	if (!multi_week_view) {
 		show_day_name = TRUE;
 		show_month_name = TRUE;
 	} else if (day == 0 || day_of_month == 1) {
@@ -251,7 +237,7 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 	if (selected) {
 		gdk_cairo_set_source_color (
 			cr, &week_view->colors[E_WEEK_VIEW_COLOR_DATES_SELECTED]);
-	} else if (week_view->multi_week_view) {
+	} else if (multi_week_view) {
 		if (today) {
 			gdk_cairo_set_source_color (
 				cr, &week_view->colors[E_WEEK_VIEW_COLOR_TODAY]);
@@ -294,7 +280,7 @@ week_view_main_item_draw_day (EWeekViewMainItem *main_item,
 	g_object_unref (layout);
 
 	/* Draw the line under the date. */
-	if (!week_view->multi_week_view) {
+	if (!multi_week_view) {
 		cairo_save (cr);
 		gdk_cairo_set_source_color (
 			cr, &week_view->colors[E_WEEK_VIEW_COLOR_GRID]);
@@ -395,13 +381,13 @@ week_view_main_item_draw (GnomeCanvasItem *canvas_item,
 	g_return_if_fail (week_view != NULL);
 
 	/* Step through each of the days. */
-	date = week_view->first_day_shown;
+	e_week_view_get_first_day_shown (week_view, &date);
 
 	/* If no date has been set, we just use Dec 1999/January 2000. */
 	if (!g_date_valid (&date))
 		g_date_set_dmy (&date, 27, 12, 1999);
 
-	num_days = week_view->multi_week_view ? week_view->weeks_shown * 7 : 7;
+	num_days = e_week_view_get_weeks_shown (week_view) * 7;
 	for (day = 0; day < num_days; day++) {
 		e_week_view_get_day_position (
 			week_view, day,
