@@ -28,14 +28,6 @@
 
 G_BEGIN_DECLS
 
-typedef struct _EMailReaderHeader EMailReaderHeader;
-
-struct _EMailReaderHeader {
-	gchar *name;
-	guint enabled : 1;
-	guint is_default : 1;
-};
-
 gboolean	e_mail_reader_confirm_delete	(EMailReader *reader);
 void		e_mail_reader_delete_folder	(EMailReader *reader,
 						 CamelFolder *folder);
@@ -68,6 +60,15 @@ void		e_mail_reader_print		(EMailReader *reader,
 void		e_mail_reader_remove_attachments
 						(EMailReader *reader);
 void		e_mail_reader_remove_duplicates	(EMailReader *reader);
+void		e_mail_reader_edit_messages	(EMailReader *reader,
+						 CamelFolder *folder,
+						 GPtrArray *uids,
+						 gboolean replace,
+						 gboolean keep_signature);
+void		e_mail_reader_forward_messages	(EMailReader *reader,
+						 CamelFolder *folder,
+						 GPtrArray *uids,
+						 EMailForwardStyle style);
 void		e_mail_reader_reply_to_message	(EMailReader *reader,
 						 CamelMimeMessage *message,
 						 EMailReplyType reply_type);
@@ -82,11 +83,6 @@ void		e_mail_reader_create_vfolder_from_selected
 						(EMailReader *reader,
 						 gint filter_type);
 
-EMailReaderHeader *
-		e_mail_reader_header_from_xml	(const gchar *xml);
-gchar *		e_mail_reader_header_to_xml	(EMailReaderHeader *header);
-void		e_mail_reader_header_free	(EMailReaderHeader *header);
-
 void		e_mail_reader_parse_message	(EMailReader *reader,
 						 CamelFolder *folder,
 						 const gchar *message_uid,
