@@ -60,7 +60,8 @@ enum {
 	E_MAIL_SIDEBAR_FOLDER_IS_TRASH			= 1 << 5,
 	E_MAIL_SIDEBAR_FOLDER_IS_VIRTUAL		= 1 << 6,
 	E_MAIL_SIDEBAR_STORE_IS_BUILTIN			= 1 << 7,
-	E_MAIL_SIDEBAR_STORE_IS_SUBSCRIBABLE		= 1 << 8
+	E_MAIL_SIDEBAR_STORE_IS_SUBSCRIBABLE		= 1 << 8,
+	E_MAIL_SIDEBAR_STORE_CAN_BE_DISABLED		= 1 << 9
 };
 
 struct _EMailSidebar {
