@@ -1967,13 +1967,18 @@ ask_drop_folder (EMFolderTree *folder_tree,
                  gboolean is_move)
 {
 	const gchar *key = is_move ? "prompt-on-folder-drop-move" : "prompt-on-folder-drop-copy";
+	const gchar *src_display_name, *des_display_name;
 	EMailSession *session;
 	GSettings *settings;
 	gchar *set_value, *src_folder_name = NULL;
+	gchar *src_folder = NULL, *des_folder = NULL;
+	CamelProvider *src_provider, *des_provider;
+	CamelStore *src_store = NULL;
 	GError *error = NULL;
 	GtkWidget *widget;
 	GtkWindow *parent;
 	gint response;
+	gboolean src_store_is_local, des_store_is_local, session_is_online;
 
 	g_return_val_if_fail (folder_tree != NULL, FALSE);
 	g_return_val_if_fail (src_folder_uri != NULL, FALSE);
@@ -2000,7 +2005,7 @@ ask_drop_folder (EMFolderTree *folder_tree,
 
 	e_mail_folder_uri_parse (
 		CAMEL_SESSION (session),
-		src_folder_uri, NULL, &src_folder_name, &error);
+		src_folder_uri, &src_store, &src_folder_name, &error);
 
 	if (error != NULL) {
 		g_warning (
@@ -2012,6 +2017,36 @@ ask_drop_folder (EMFolderTree *folder_tree,
 		return FALSE;
 	}
 
+	session_is_online = camel_session_get_online (CAMEL_SESSION (session));
+
+	src_provider = camel_service_get_provider (CAMEL_SERVICE (src_store));
+	src_store_is_local = (src_provider->flags & CAMEL_PROVIDER_IS_LOCAL) != 0;
+	src_display_name = camel_service_get_display_name (CAMEL_SERVICE (src_store));
+	src_folder = g_strdup_printf ("%s: %s", src_display_name, src_folder_name);
+
+	des_provider = camel_service_get_provider (CAMEL_SERVICE (des_store));
+	des_store_is_local = (des_provider->flags & CAMEL_PROVIDER_IS_LOCAL) != 0;
+	des_display_name = camel_service_get_display_name (CAMEL_SERVICE (des_store));
+	des_folder = g_strdup_printf ("%s: %s", des_display_name, des_full_name);
+
+	if (!session_is_online && (!src_store_is_local || !des_store_is_local)) {
+		EAlertSink *alert_sink;
+
+		alert_sink = em_folder_tree_get_alert_sink (folder_tree);
+		e_alert_submit (
+			alert_sink,
+			"mail:online-operation",
+			src_store_is_local ? des_folder : src_folder,
+			NULL);
+		g_free (src_folder_name);
+		g_free (src_folder);
+		g_free (des_folder);
+		g_object_unref (src_store);
+		g_object_unref (settings);
+
+		return FALSE;
+	}
+
 	parent = NULL;
 	widget = gtk_widget_get_toplevel (GTK_WIDGET (folder_tree));
 	if (widget && gtk_widget_is_toplevel (widget) && GTK_IS_WINDOW (widget))
@@ -2021,7 +2056,7 @@ ask_drop_folder (EMFolderTree *folder_tree,
 		parent,
 		is_move ? "mail:ask-folder-drop-move" : "mail:ask-folder-drop-copy",
 		src_folder_name,
-		des_full_name && *des_full_name ? des_full_name :
+		des_full_name && *des_full_name ? des_folder :
 			camel_service_get_display_name (CAMEL_SERVICE (des_store)),
 		NULL);
 	response = gtk_dialog_run (GTK_DIALOG (widget));
@@ -2033,6 +2068,9 @@ ask_drop_folder (EMFolderTree *folder_tree,
 		g_settings_set_string (settings, key, "never");
 
 	g_free (src_folder_name);
+	g_free (src_folder);
+	g_free (des_folder);
+	g_object_unref (src_store);
 	g_object_unref (settings);
 
 	return response == GTK_RESPONSE_YES || response == GTK_RESPONSE_OK;
