@@ -56,7 +56,8 @@ enum {
 	E_BOOK_SHELL_SIDEBAR_PRIMARY_SOURCE_IS_REMOVABLE	= 1 << 2,
 	E_BOOK_SHELL_SIDEBAR_PRIMARY_SOURCE_IS_REMOTE_CREATABLE	= 1 << 3,
 	E_BOOK_SHELL_SIDEBAR_PRIMARY_SOURCE_IS_REMOTE_DELETABLE	= 1 << 4,
-	E_BOOK_SHELL_SIDEBAR_PRIMARY_SOURCE_IN_COLLECTION	= 1 << 5
+	E_BOOK_SHELL_SIDEBAR_PRIMARY_SOURCE_IN_COLLECTION	= 1 << 5,
+	E_BOOK_SHELL_SIDEBAR_SOURCE_SUPPORTS_REFRESH		= 1 << 6
 };
 
 struct _EBookShellSidebar {
