@@ -32,12 +32,22 @@ action_calendar_memopad_forward_cb (GtkAction *action,
                                     ECalShellView *cal_shell_view)
 {
 	ECalShellContent *cal_shell_content;
+	EShell *shell;
+	EShellView *shell_view;
+	EShellWindow *shell_window;
+	ESourceRegistry *registry;
 	EMemoTable *memo_table;
 	ECalModelComponent *comp_data;
 	ECalComponent *comp;
 	icalcomponent *clone;
 	GSList *list;
 
+	shell_view = E_SHELL_VIEW (cal_shell_view);
+	shell_window = e_shell_view_get_shell_window (shell_view);
+	shell = e_shell_window_get_shell (shell_window);
+
+	registry = e_shell_get_registry (shell);
+
 	cal_shell_content = cal_shell_view->priv->cal_shell_content;
 	memo_table = e_cal_shell_content_get_memo_table (cal_shell_content);
 
@@ -52,7 +62,7 @@ action_calendar_memopad_forward_cb (GtkAction *action,
 	e_cal_component_set_icalcomponent (comp, clone);
 
 	itip_send_comp (
-		E_CAL_COMPONENT_METHOD_PUBLISH, comp,
+		registry, E_CAL_COMPONENT_METHOD_PUBLISH, comp,
 		comp_data->client, NULL, NULL, NULL, TRUE, FALSE);
 
 	g_object_unref (comp);
@@ -393,6 +403,7 @@ e_cal_shell_view_memopad_open_memo (ECalShellView *cal_shell_view,
 	EShell *shell;
 	EShellView *shell_view;
 	EShellWindow *shell_window;
+	ESourceRegistry *registry;
 	CompEditor *editor;
 	CompEditorFlags flags = 0;
 	ECalComponent *comp;
@@ -406,6 +417,8 @@ e_cal_shell_view_memopad_open_memo (ECalShellView *cal_shell_view,
 	shell_window = e_shell_view_get_shell_window (shell_view);
 	shell = e_shell_window_get_shell (shell_window);
 
+	registry = e_shell_get_registry (shell);
+
 	uid = icalcomponent_get_uid (comp_data->icalcomp);
 	editor = comp_editor_find_instance (uid);
 
@@ -419,7 +432,7 @@ e_cal_shell_view_memopad_open_memo (ECalShellView *cal_shell_view,
 	if (e_cal_component_has_organizer (comp))
 		flags |= COMP_EDITOR_IS_SHARED;
 
-	if (itip_organizer_is_user (comp, comp_data->client))
+	if (itip_organizer_is_user (registry, comp, comp_data->client))
 		flags |= COMP_EDITOR_USER_ORG;
 
 	editor = memo_editor_new (comp_data->client, shell, flags);
